;********************************************************
;
;              Decorative light Controller
;         for checking BCD switch intput operation
;
;                                 Device : PIC16F628
;                                 Clock  : 4MHz
;                                 Author : Seiichi Inoue
;********************************************************
;This software is for checking input operation of BCD
;switches.The ON timing of the RA ports will be changed
;by the setting of switches.

        list            p=pic16f628
        include         p16f628.inc
 __config _hs_osc & _wdt_off & _pwrte_on & _lvp_off & _boden_off
        errorlevel      -302    ;Eliminate bank warning

;**************  Operating condition  *******************

;*** Frequency of AC power. 50Hz or 60Hz
hz      equ    d'50'            ;Except 50Hz assume 60Hz

;****************  Label Definition  ********************
        cblock  h'20'
tmr0_value                      ;Value set to TMR0
tmr0_index                      ;TMR0 index
porta_work                      ;PORTA work area
portb_work                      ;PORTB work area
triac1                          ;TRIAC1 data
triac2                          ;TRIAC2 data
triac3                          ;TRIAC3 data
triac4                          ;TRIAC4 data
triac5                          ;TRIAC5 data
sw1_mode                        ;BCD SW1 mode
sw2_mode                        ;BCD SW2 mode
sw3_mode                        ;BCD SW3 mode
sw4_mode                        ;BCD SW4 mode
sw5_mode                        ;BCD SW5 mode
sw_count                        ;BCD SW scan counter
p_count                         ;Pattern trigger counter
s_count                         ;Switch read wait counter

;TRIAC1 work area
t1_s                            ;Start delay
t1_r                            ;Rising up
t1_b                            ;Bright
t1_f                            ;Falling down
t1_d                            ;Dim
t1_proc                         ;Process No.
t1_count                        ;Counter
t1_flag                         ;Counter Flag
t1_index                        ;Index
t1_subindex                     ;Sub Index

;TRIAC2 work area
t2_s                            ;Start delay
t2_r                            ;Rising up
t2_b                            ;Bright
t2_f                            ;Falling down
t2_d                            ;Dim
t2_proc                         ;Process No.
t2_count                        ;Counter
t2_flag                         ;Counter Flag
t2_index                        ;Index
t2_subindex                     ;Sub Index

;TRIAC3 work area
t3_s                            ;Start delay
t3_r                            ;Rising up
t3_b                            ;Bright
t3_f                            ;Falling down
t3_d                            ;Dim
t3_proc                         ;Process No.
t3_count                        ;Counter
t3_flag                         ;Counter Flag
t3_index                        ;Index
t3_subindex                     ;Sub Index

;TRIAC4 work area
t4_s                            ;Start delay
t4_r                            ;Rising up
t4_b                            ;Bright
t4_f                            ;Falling down
t4_d                            ;Dim
t4_proc                         ;Process No.
t4_count                        ;Counter
t4_flag                         ;Counter Flag
t4_index                        ;Index
t4_subindex                     ;Sub Index

;TRIAC5 work area
t5_s                            ;Start delay
t5_r                            ;Rising up
t5_b                            ;Bright
t5_f                            ;Falling down
t5_d                            ;Dim
t5_proc                         ;Process No.
t5_count                        ;Counter
t5_flag                         ;Counter Flag
t5_index                        ;Index
t5_subindex                     ;Sub Index
        endc

ra0     equ     00              ;RA0 bit
ra1     equ     01              ;RA1 bit
ra2     equ     02              ;RA2 bit
ra3     equ     03              ;RA3 bit
ra4     equ     04              ;RA4 bit


;***************** Program Start ************************
        org     0               ;Reset Vector
        goto    init
        org     4               ;Interrupt Vector
        goto    int

;******************* Initial Process ********************
init
;*** Set Port mode
        bsf     status,rp0      ;Change to Bank1
        movlw   b'00000000'     ;RA4-0:OUT mode
        movwf   trisa           ;Set TRISA reg
        movlw   b'00011111'     ;RB4-0:IN RB7-5:OUT mode
        movwf   trisb           ;Set TRISB reg

;*** Set Option reg
        movlw   b'01000001'     ;RBPU=ON,INTEDG=ON,PS=1:4
        movwf   option_reg      ;Set OPTION_REG

;*** Set PIE1 reg
        movlw   b'00000001'     ;TMR1IE=ON
        movwf   pie1            ;Set PIE1 reg
        bcf     status,rp0      ;Change to Bank0

;*** Set CMCON reg
        movlw   b'00000111'     ;RA port to digital
        movwf   cmcon           ;Set CMCON reg

;*** Set T1CON reg
        movlw   b'00000001'     ;T1CKPS=1:1,TMR1=ON
        movwf   t1con           ;Set T1CON reg

;*** Set TMR0 reg
        movlw   d'50'           ;Set check data
        sublw   hz              ;Check frequency
        btfss   status,z        ;50Hz ?
        goto    hz60            ;No.
        movlw   d'9'            ;256-1ms/0.004ms=6+Adj
        goto    tmr0set
hz60
        movlw   d'51'           ;256-833us/4us=48+Adj
tmr0set
        movwf   tmr0_value      ;Save TMR0 value
        movwf   tmr0            ;Set TMR0

;*** Set TMR1H,TMR1L reg (50ms)
        movlw   h'3c'           ;65536-50000us=15536+Adj
        movwf   tmr1h           ;d'15550' = h'3CBE'
        movlw   h'be'           ;Set TMR1H = h'3C'
        movwf   tmr1l           ;Set TMR1L = h'BE'

;*** Other registers and work area initialization
        clrf    porta           ;Clear PORTA
        clrf    portb           ;Clear PORTB
        clrf    tmr0_index      ;Clear TMR0 index
        clrf    porta_work      ;Clear PORTA work area
        movlw   d'10'           ;Set TRIAC OFF data
        movwf   triac1          ;Set TRIAC1 data
        movwf   triac2          ;Set TRIAC2 data
        movwf   triac3          ;Set TRIAC3 data
        movwf   triac4          ;Set TRIAC4 data
        movwf   triac5          ;Set TRIAC5 data
        movlw   d'2'            ;Set value
        movwf   p_count         ;Set pattten trigger cnt
        clrf    sw_count        ;Clear switch scan count
        clrf    sw1_mode        ;Clear switch mode TRIAC1
        clrf    sw2_mode        ;Clear switch mode TRIAC2
        clrf    sw3_mode        ;Clear switch mode TRIAC3
        clrf    sw4_mode        ;Clear switch mode TRIAC4
        clrf    sw5_mode        ;Clear switch mode TRIAC5
        clrf    t1_proc         ;Clear process for TRIAC1
        clrf    t2_proc         ;Clear process for TRIAC2
        clrf    t3_proc         ;Clear process for TRIAC3
        clrf    t4_proc         ;Clear process for TRIAC4
        clrf    t5_proc         ;Clear process for TRIAC5

;*** Set interrupt condition
        movlw   b'11110000'     ;GIE,PEIE,TOIE,INTE=1
        movwf   intcon          ;Interruption enable

        goto    $               ;Wait interruption

int
;*********** Interruption process beginning *************
        btfsc   intcon,intf     ;RB0 interruption ?
        goto    rb0_int         ;Jump to RB0 interruption
        btfsc   intcon,t0if     ;TMR0 time out ?
        goto    tmr0_int        ;Jump to TMR0 time-out
        btfsc   pir1,tmr1if     ;TMR1 time out ?
        goto    tmr1_int        ;Jump to TMR1 time-out
        goto    init            ;Reset(Illegal interrupt)

int_end
;************* END of interruption process **************
        retfie

rb0_int
;*************** RB0 interruption process ***************
        bcf     intcon,intf     ;Clear RB0 int flag
        bcf     intcon,t0if     ;Clear TMR0 int flag
        movfw   tmr0_value      ;Set timer value
        movwf   tmr0            ;Set TMR0
        clrf    tmr0_index      ;Clear TMR0 index
        clrf    porta_work      ;Clear PORTA work area
        clrf    porta           ;Clear PORTA

        goto    int_end         ;End of RB0 interruption

tmr0_int
;************** TMR0 interruption process ***************
        bcf     intcon,t0if     ;Clear TMR0 int flag
        movfw   tmr0_value      ;Set timer value
        movwf   tmr0            ;Set TMR0

        incf    tmr0_index,f    ;TMR0 index + 1
tmr0_triac1
        movfw   triac1          ;Read TRIAC1 data
        subwf   tmr0_index,w    ;Check TMR0 index
        btfss   status,c        ;TRIAC1 =< index ?
        goto    tmr0_triac2     ;No. Next
        bsf     porta_work,ra0  ;Set TRIAC1 ON
tmr0_triac2
        movfw   triac2          ;Read TRIAC2 data
        subwf   tmr0_index,w    ;Check TMR0 index
        btfss   status,c        ;TRIAC2 =< index ?
        goto    tmr0_triac3     ;No. Next
        bsf     porta_work,ra1  ;Set TRIAC2 ON
tmr0_triac3
        movfw   triac3          ;Read TRIAC3 data
        subwf   tmr0_index,w    ;Check TMR0 index
        btfss   status,c        ;TRIAC3 =< index ?
        goto    tmr0_triac4     ;No. Next
        bsf     porta_work,ra2  ;Set TRIAC3 ON
tmr0_triac4
        movfw   triac4          ;Read TRIAC4 data
        subwf   tmr0_index,w    ;Check TMR0 index
        btfss   status,c        ;TRIAC4 =< index ?
        goto    tmr0_triac5     ;No. Next
        bsf     porta_work,ra3  ;Set TRIAC4 ON
tmr0_triac5
        movfw   triac5          ;Read TRIAC5 data
        subwf   tmr0_index,w    ;Check TMR0 index
        btfss   status,c        ;TRIAC5 =< index ?
        goto    tmr0_end        ;No. go to end
        bsf     porta_work,ra4  ;Set TRIAC5 ON
tmr0_end
        movfw   porta_work      ;Read PORTA work area
        movwf   porta           ;Write PORTA

        goto    int_end         ;End of TMR0 interruption

tmr1_int
;***** Timer1 interruption process (50ms interval) ******
        bcf     pir1,tmr1if     ;Clear TMR1 int flag
        movlw   h'3c'           ;Set TMR1H value
        movwf   tmr1h           ;Set TMR1H
        movlw   h'be'           ;Set TMR1L value
        movwf   tmr1l           ;Set TMR1L

;*** BCD switch scanning
sw1
        movfw   sw_count        ;Read switch scan position
        btfss   status,z        ;Switch 1 ?
        goto    sw2             ;No. Next
        movlw   b'00000000'     ;Set scan data
        call    sw_mode_sub     ;Call SW mode pick up
        movfw   portb_work      ;Read PORTB data
        subwf   sw1_mode,w      ;Check PORTB data
        btfsc   status,z        ;PORTB changed ?
        goto    sw_end          ;No. Jump to sw scan end
        clrf    t1_proc         ;Set process to 0
        movfw   portb_work      ;Read PORTB data
        movwf   sw1_mode        ;Save SW1 mode
        goto    sw_end          ;Jump to sw scan end
sw2
        movlw   d'1'            ;Set check data
        subwf   sw_count,w      ;Check position data
        btfss   status,z        ;Switch 2 ?
        goto    sw3             ;No. Next
        movlw   b'00100000'     ;Set scan data
        call    sw_mode_sub     ;Call SW mode pick up
        movfw   portb_work      ;Read PORTB data
        subwf   sw2_mode,w      ;Check PORTB data
        btfsc   status,z        ;PORTB changed ?
        goto    sw_end          ;No. Jump to sw scan end
        clrf    t2_proc         ;Set process to 0
        movfw   portb_work      ;Read PORTB data
        movwf   sw2_mode        ;Save SW2 mode
        goto    sw_end          ;Jump to sw scan end
sw3
        movlw   d'2'            ;Set check data
        subwf   sw_count,w      ;Check position data
        btfss   status,z        ;Switch 3 ?
        goto    sw4             ;No. Next
        movlw   b'01000000'     ;Set scan data
        call    sw_mode_sub     ;Call SW mode pick up
        movfw   portb_work      ;Read PORTB data
        subwf   sw3_mode,w      ;Check PORTB data
        btfsc   status,z        ;PORTB changed ?
        goto    sw_end          ;No. Jump to sw scan end
        clrf    t3_proc         ;Set process to 0
        movfw   portb_work      ;Read PORTB data
        movwf   sw3_mode        ;Save SW3 mode
        goto    sw_end          ;Jump to sw scan end
sw4
        movlw   d'3'            ;Set check data
        subwf   sw_count,w      ;Check position data
        btfss   status,z        ;Switch 4 ?
        goto    sw5             ;No. Next
        movlw   b'01100000'     ;Set scan data
        call    sw_mode_sub     ;Call SW mode pick up
        movfw   portb_work      ;Read PORTB data
        subwf   sw4_mode,w      ;Check PORTB data
        btfsc   status,z        ;PORTB changed ?
        goto    sw_end          ;No. Jump to sw scan end
        clrf    t4_proc         ;Set process to 0
        movfw   portb_work      ;Read PORTB data
        movwf   sw4_mode        ;Save SW4 mode
        goto    sw_end          ;Jump to sw scan end
sw5
        movlw   d'4'            ;Set check data
        subwf   sw_count,w      ;Check position data
        btfss   status,z        ;Switch 5 ?
        goto    sw_end          ;No. Jump to sw scan end
        movlw   b'10000000'     ;Set scan data
        call    sw_mode_sub     ;Call SW mode pick up
        movfw   portb_work      ;Read PORTB data
        subwf   sw5_mode,w      ;Check PORTB data
        btfsc   status,z        ;PORTB changed ?
        goto    sw_end          ;No. Jump to sw scan end
        clrf    t5_proc         ;Set process to 0
        movfw   portb_work      ;Read PORTB data
        movwf   sw5_mode        ;Save SW5 mode

sw_end
        incf    sw_count,f      ;Switch position + 1
        movfw   sw_count        ;Read switch counter
        sublw   d'4'            ;Set check data
        btfss   status,c        ;Count over ?
        clrf    sw_count        ;Clear switch counter
        goto    ptn_cnt         ;Jump to pattern check

sw_mode_sub
        movwf   portb           ;Write scan data
        movlw   d'2'            ;Set 7us wait
        movwf   s_count         ;Save count
loop
        decfsz  s_count,f       ;7us ?
        goto    loop            ;No.
        movfw   portb           ;Read PORTB
        movwf   portb_work      ;Save PORTB data
        rrf     portb_work,f    ;Rotate Right
        movlw   b'00001111'     ;Set mask
        xorwf   portb_work,f    ;Reverse over BCD data
        andwf   portb_work,f    ;Pick up BCD data
        return

;*** Check pattern process timing
ptn_cnt
        decfsz  p_count,f       ;100ms ?
        goto    int_end         ;No. Next timing
        movlw   d'2'            ;Set trigger count data
        movwf   p_count         ;Set pattten trigger

;************** Setting SW-MODE to TRIAC ****************
        movfw   sw1_mode        ;Read SW1 data
        movwf   triac1          ;Set TRIAC1
        movfw   sw2_mode        ;Read SW2 data
        movwf   triac2          ;Set TRIAC2
        movfw   sw3_mode        ;Read SW3 data
        movwf   triac3          ;Set TRIAC3
        movfw   sw4_mode        ;Read SW4 data
        movwf   triac4          ;Set TRIAC4
        movfw   sw5_mode        ;Read SW5 data
        movwf   triac5          ;Set TRIAC5

        goto    int_end         ;End of TMR1 interruption

;********************************************************
;          END of Decorative light Controller
;********************************************************

        end
